package com.god.base.server.websocket;

import com.alibaba.fastjson.JSON;

import com.god.common.core.utils.SpringUtils;
import lombok.AllArgsConstructor;
import lombok.NoArgsConstructor;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.math.BigDecimal;
import java.util.*;

import com.god.common.redis.service.RedisService;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArraySet;

/**
 * Description:
 *
 * @Author: sun-cool-boy
 * @Date: 2023/11/29
 * @info:
 */
@Slf4j
@Component
@ServerEndpoint("/websocket/{vin}")
public class WebSocketServer{

    /**
     * 注入redis
     */
    @Autowired
    private RedisService redisService = SpringUtils.getBean(RedisService.class);
//    @Autowired
//    public WebSocketServer(RedisService redisService) {
//        this.redisService = redisService;
//    }
    /**
     *与某个客户端建立连接会话 通过session给客户端发送消息
     */
    private Session session;

    /**
     * 客户端标识 用vin
     */
    private String vin;

    /**
     * redis轨迹标识
     */
    public static final String ALL_LOCUS_INFO = "all_locus_info";

    /**
     * concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
     * 虽然@Component默认是单例模式的，但springboot还是会为每个websocket连接初始化一个bean，所以可以用一个静态set保存起来。
     * 注：底下WebSocketServer是当前类名
     */
    private static final CopyOnWriteArraySet<WebSocketServer> webSocketServers = new CopyOnWriteArraySet<>();

    /**
     * 定义ConcurrentHashMap负责存储客户端信息 vin为键 session为值
     */
    private static final ConcurrentHashMap<String,Session> map = new  ConcurrentHashMap<>();



    /**
     * 连接事件
     * @param session
     * @param vin
     */
    @OnOpen
    public void onOpen(Session session, @PathParam(value = "vin") String vin){
        try {
            this.session = session;
            this.vin = vin;
            webSocketServers.add(this);
            map.put(vin,session);
            log.info("【webSocket消息】 有新的连接，总数为："+webSocketServers.size());
            if (webSocketServers.size() == 1) {
                timedMsg();
            }
        }catch (Exception e){
            log.info("【webSocket消息】 客户端 {} 连接失败",vin);
            e.printStackTrace();
        }
    }


    public void timedMsg(){
        new Thread(() -> {
            while (!map.isEmpty()) {
                try {
                    map.forEach((key,session) -> {
                        if (session.isOpen()){
                            Map<String, Object> cacheMap = redisService.getCacheMap(ALL_LOCUS_INFO);
                            ArrayList<ArrayList<BigDecimal>> orDefault = (ArrayList<ArrayList<BigDecimal>>) cacheMap.getOrDefault(key, new ArrayList<ArrayList<BigDecimal>>());
                            ArrayList<BigDecimal> bigDecimals = orDefault.get(orDefault.size() - 1);
                            log.info("【webSocket消息】服务端发起了广播消息：{}",bigDecimals.toString());
                            sendOneMessage(key, JSON.toJSONString(bigDecimals));
                        }
                    });
                    Thread.sleep(1800);
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
            }
        }).start();
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose(){
        webSocketServers.remove(this);
        map.remove(this.vin);
        log.info("【websocket消息】连接断开 客户端id {} , 剩余客户端连接 {}",this.vin,webSocketServers.size());
    }

    /**
     * 收到客户端消息后调用的方法
     * @param msg
     */
    @OnMessage
    public void onMessage(String msg){
        log.info("【webSocket消息】收到客户端消息:" + msg);
    }

    /**
     * 异常事件
     * @param throwable
     */
    @OnError
    public void onError(Throwable throwable){
        log.error("【webSocket消息】客户端错误 " + throwable.getMessage());
        throwable.printStackTrace();
    }

    /**
     * 广播消息 所有客户端都能收到消息
     * @param msg
     */
    public void sendAllMessage(String msg){
        log.info("【webSocket消息】服务端发起了广播消息：{}",msg);
        webSocketServers.forEach(obj -> {
            if (obj.session.isOpen()){
                obj.session.getAsyncRemote().sendText(msg);
            }
        });
    }

    /**
     * 单客户端接收消息
     * @param vin
     * @param msg
     */
    public void sendOneMessage(String vin , String msg){
        log.info("【webSocket消息】向客户端 {} 发送消息 , 消息内容 {}",vin,msg);
        Session session = map.get(vin);
        if (null != session && session.isOpen()){
            session.getAsyncRemote().sendText(JSON.toJSONString(msg));
        }
    }


    /**
     * 多个指定客户端接收消息
     * @param vinArr
     * @param msg
     */
    public void sendManyMessage(String [] vinArr,String msg){
        log.info("【webSocket消息】向客户端 {} 发送消息 , 消息内容 {}", Arrays.toString(vinArr),msg);
        for (String vin : vinArr) {
            Session session = map.get(vin);
            session.getAsyncRemote().sendText(JSON.toJSONString(msg));
        }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        WebSocketServer that = (WebSocketServer) o;
        return Objects.equals(session, that.session) && Objects.equals(vin, that.vin);
    }

    @Override
    public int hashCode() {
        return Objects.hash(session, vin);
    }
}
